@@ -24,8 +24,20 @@ from functools import cached_property
 
 import gi  # type: ignore
 
-gi.require_version("GIRepository", "2.0")
-from gi.repository import GIRepository  # type: ignore
+try:
+    gi.require_version("GIRepository", "3.0")
+    from gi.repository import GIRepository  # type: ignore
+
+    _repo = GIRepository.Repository()
+except ValueError:
+    # We can remove this once we can bump the minimum dependencies
+    # to glib 2.80 and pygobject 3.52
+    # dependency('glib-2.0', version: '>= 2.80.0')
+    # dependency('girepository-2.0', version: '>= 2.80.0')
+    gi.require_version("GIRepository", "2.0")
+    from gi.repository import GIRepository  # type: ignore
+
+    _repo = GIRepository.Repository
 
 from . import typelib, xml_reader
 from .errors import CompileError, CompilerBugError
@@ -42,7 +54,7 @@ def add_typelib_search_path(path: str):
 
 
 def get_namespace(namespace: str, version: str) -> "Namespace":
-    search_paths = [*GIRepository.Repository.get_search_path(), *_user_search_paths]
+    search_paths = [*_repo.get_search_path(), *_user_search_paths]
 
     filename = f"{namespace}-{version}.typelib"
 
@@ -74,7 +86,7 @@ def get_available_namespaces() -> T.List[T.Tuple[str, str]]:
         return _available_namespaces
 
     search_paths: list[str] = [
-        *GIRepository.Repository.get_search_path(),
+        *_repo.get_search_path(),
         *_user_search_paths,
     ]
 
@@ -200,6 +212,10 @@ class ArrayType(GirType):
     def assignable_to(self, other: GirType) -> bool:
         return isinstance(other, ArrayType) and self._inner.assignable_to(other._inner)
 
+    @property
+    def inner(self) -> GirType:
+        return self._inner
+
     @property
     def name(self) -> str:
         return self._inner.name + "[]"
@@ -332,6 +348,17 @@ class GirNode:
     def available_in(self) -> str:
         return self.xml.get("version")
 
+    @cached_property
+    def detail(self) -> T.Optional[str]:
+        try:
+            el = self.xml.get_elements("doc")
+            if len(el) == 1:
+                return el[0].cdata.strip().partition("\n")[0]
+            else:
+                return None
+        except:
+            return None
+
     @cached_property
     def doc(self) -> T.Optional[str]:
         sections = []
@@ -440,10 +467,13 @@ class Signature(GirNode):
         return result
 
     @cached_property
-    def return_type(self) -> GirType:
-        return self.get_containing(Repository)._resolve_type_id(
-            self.tl.SIGNATURE_RETURN_TYPE
-        )
+    def return_type(self) -> T.Optional[GirType]:
+        if self.tl.SIGNATURE_RETURN_TYPE == 0:
+            return None
+        else:
+            return self.get_containing(Repository)._resolve_type_id(
+                self.tl.SIGNATURE_RETURN_TYPE
+            )
 
 
 class Signal(GirNode):
@@ -463,7 +493,10 @@ class Signal(GirNode):
         args = ", ".join(
             [f"{a.type.full_name} {a.name}" for a in self.gir_signature.args]
         )
-        return f"signal {self.container.full_name}::{self.name} ({args})"
+        result = f"signal {self.container.full_name}::{self.name} ({args})"
+        if self.gir_signature.return_type is not None:
+            result += f" -> {self.gir_signature.return_type.full_name}"
+        return result
 
     @property
     def online_docs(self) -> T.Optional[str]:
@@ -534,7 +567,7 @@ class Interface(GirNode, GirType):
     @property
     def online_docs(self) -> T.Optional[str]:
         if ns := self.get_containing(Namespace).online_docs:
-            return f"{ns}interface.{self.name}.html"
+            return f"{ns}iface.{self.name}.html"
         else:
             return None
 
@@ -706,7 +739,7 @@ class TemplateType(GirType):
             # we don't know the template type's interfaces, assume yes
             return True
         elif self.parent is None or isinstance(self.parent, ExternType):
-            return isinstance(other, Class)
+            return isinstance(other, Class) or isinstance(other, ExternType)
         else:
             return self.parent.assignable_to(other)
 
@@ -875,22 +908,22 @@ class Namespace(GirNode):
             if isinstance(entry, Class)
         }
 
-    @cached_property
-    def interfaces(self) -> T.Mapping[str, Interface]:
-        return {
-            name: entry
-            for name, entry in self.entries.items()
-            if isinstance(entry, Interface)
-        }
-
     def get_type(self, name) -> T.Optional[GirType]:
         """Gets a type (class, interface, enum, etc.) from this namespace."""
         return self.entries.get(name)
 
     def get_type_by_cname(self, cname: str) -> T.Optional[GirType]:
         """Gets a type from this namespace by its C name."""
+        for basic in _BASIC_TYPES.values():
+            if basic.glib_type_name == cname:
+                return basic()
+
         for item in self.entries.values():
-            if hasattr(item, "cname") and item.cname == cname:
+            if (
+                hasattr(item, "cname")
+                and item.cname is not None
+                and item.cname == cname
+            ):
                 return item
         return None
 
@@ -898,13 +931,8 @@ class Namespace(GirNode):
         """Looks up a type in the scope of this namespace (including in the
         namespace's dependencies)."""
 
-        if type_name in _BASIC_TYPES:
-            return _BASIC_TYPES[type_name]()
-        elif "." in type_name:
-            ns, name = type_name.split(".", 1)
-            return self.get_containing(Repository).get_type(name, ns)
-        else:
-            return self.get_type(type_name)
+        ns, name = type_name.split(".", 1)
+        return self.get_containing(Repository).get_type(name, ns)
 
     @property
     def online_docs(self) -> T.Optional[str]:
@@ -923,7 +951,7 @@ class Repository(GirNode):
                 self.includes = {
                     name: get_namespace(name, version) for name, version in deps
                 }
-            except:
+            except:  # pragma: no cover
                 raise CompilerBugError(f"Failed to load dependencies.")
         else:
             self.includes = {}
@@ -931,12 +959,6 @@ class Repository(GirNode):
     def get_type(self, name: str, ns: str) -> T.Optional[GirType]:
         return self.lookup_namespace(ns).get_type(name)
 
-    def get_type_by_cname(self, name: str) -> T.Optional[GirType]:
-        for ns in [self.namespace, *self.includes.values()]:
-            if type := ns.get_type_by_cname(name):
-                return type
-        return None
-
     def lookup_namespace(self, ns: str):
         """Finds a namespace among this namespace's dependencies."""
         if ns == self.namespace.name:
