@@ -88,6 +88,10 @@ class Template(Object):
             f"Only one template may be defined per file, but this file contains {len(self.parent.children[Template])}",
         )
 
+    @docs("id")
+    def ref_docs(self):
+        return get_docs_section("Syntax Template")
+
 
 @decompiler("template")
 def decompile_template(ctx: DecompileCtx, gir, klass, parent=None):
@@ -97,8 +101,9 @@ def decompile_template(ctx: DecompileCtx, gir, klass, parent=None):
         else:
             return "$" + cname
 
-    ctx.print(f"template {class_name(klass)} : {class_name(parent)} {{")
-
-    ctx.template_class = klass
+    if parent is None:
+        ctx.print(f"template {class_name(klass)} {{")
+    else:
+        ctx.print(f"template {class_name(klass)} : {class_name(parent)} {{")
 
     return ctx.type_by_cname(klass) or ctx.type_by_cname(parent)
