@@ -25,6 +25,7 @@ import sys
 import typing as T
 
 from . import formatter, interactive_port, parser, tokenizer
+from .decompiler import decompile_string
 from .errors import CompileError, CompilerBugError, PrintableError, report_bug
 from .gir import add_typelib_search_path
 from .lsp import LanguageServer
@@ -90,12 +91,28 @@ class BlueprintApp:
             default=2,
             type=int,
         )
+        format.add_argument(
+            "-n",
+            "--no-diff",
+            help="Do not print a full diff of the changes",
+            default=False,
+            action="store_true",
+        )
         format.add_argument(
             "inputs",
             nargs="+",
             metavar="filenames",
         )
 
+        decompile = self.add_subcommand(
+            "decompile", "Convert .ui XML files to blueprint", self.cmd_decompile
+        )
+        decompile.add_argument("--output", dest="output", default="-")
+        decompile.add_argument("--typelib-path", nargs="?", action="append")
+        decompile.add_argument(
+            "input", metavar="filename", default=sys.stdin, type=argparse.FileType("r")
+        )
+
         port = self.add_subcommand("port", "Interactive porting tool", self.cmd_port)
 
         lsp = self.add_subcommand(
@@ -221,23 +238,25 @@ class BlueprintApp:
                         file.write(formatted_str)
                         happened = "Formatted"
 
-                    diff_lines = []
-                    a_lines = data.splitlines(keepends=True)
-                    b_lines = formatted_str.splitlines(keepends=True)
+                    if not opts.no_diff:
+                        diff_lines = []
+                        a_lines = data.splitlines(keepends=True)
+                        b_lines = formatted_str.splitlines(keepends=True)
 
-                    for line in difflib.unified_diff(
-                        a_lines, b_lines, fromfile=file.name, tofile=file.name, n=5
-                    ):
-                        # Work around https://bugs.python.org/issue2142
-                        # See:
-                        # https://www.gnu.org/software/diffutils/manual/html_node/Incomplete-Lines.html
-                        if line[-1] == "\n":
-                            diff_lines.append(line)
-                        else:
-                            diff_lines.append(line + "\n")
-                            diff_lines.append("\\ No newline at end of file\n")
+                        for line in difflib.unified_diff(
+                            a_lines, b_lines, fromfile=file.name, tofile=file.name, n=5
+                        ):
+                            # Work around https://bugs.python.org/issue2142
+                            # See:
+                            # https://www.gnu.org/software/diffutils/manual/html_node/Incomplete-Lines.html
+                            if line[-1] == "\n":
+                                diff_lines.append(line)
+                            else:
+                                diff_lines.append(line + "\n")
+                                diff_lines.append("\\ No newline at end of file\n")
+
+                        print("".join(diff_lines))
 
-                    print("".join(diff_lines))
                     to_print = Colors.BOLD
                     if errored:
                         to_print += f"{Colors.RED}Skipped {file.name}: Will not overwrite file with compile errors"
@@ -291,6 +310,24 @@ class BlueprintApp:
         if panic:
             sys.exit(1)
 
+    def cmd_decompile(self, opts):
+        if opts.typelib_path != None:
+            for typelib_path in opts.typelib_path:
+                add_typelib_search_path(typelib_path)
+
+        data = opts.input.read()
+        try:
+            decompiled = decompile_string(data)
+
+            if opts.output == "-":
+                print(decompiled)
+            else:
+                with open(opts.output, "w") as file:
+                    file.write(decompiled)
+        except PrintableError as e:
+            e.pretty_print(opts.input.name, data, stream=sys.stderr)
+            sys.exit(1)
+
     def cmd_lsp(self, opts):
         langserv = LanguageServer()
         langserv.run()
