@@ -1,15 +1,22 @@
-import os, sys
+import os
+import sys
+
 from pythonfuzz.main import PythonFuzz
 
 from blueprintcompiler.outputs.xml import XmlOutput
 
 sys.path.insert(0, os.path.dirname(os.path.dirname(__file__)))
 
-from blueprintcompiler import tokenizer, parser, decompiler, gir
+from blueprintcompiler import decompiler, gir, parser, tokenizer, utils
 from blueprintcompiler.completions import complete
-from blueprintcompiler.errors import PrintableError, MultipleErrors, CompileError, CompilerBugError
+from blueprintcompiler.errors import (
+    CompileError,
+    CompilerBugError,
+    MultipleErrors,
+    PrintableError,
+)
 from blueprintcompiler.tokenizer import Token, TokenType, tokenize
-from blueprintcompiler import utils
+
 
 @PythonFuzz
 def fuzz(buf):
@@ -20,7 +27,7 @@ def fuzz(buf):
         ast, errors, warnings = parser.parse(tokens)
 
         xml = XmlOutput()
-        if errors is None and len(ast.errors) == 0:
+        if errors is None and ast is not None:
             xml.emit(ast)
     except CompilerBugError as e:
         raise e
@@ -29,6 +36,7 @@ def fuzz(buf):
     except UnicodeDecodeError:
         pass
 
+
 if __name__ == "__main__":
     # Make sure Gtk 4.0 is accessible, otherwise every test will fail on that
     # and nothing interesting will be tested
