@@ -18,6 +18,7 @@
 # SPDX-License-Identifier: LGPL-3.0-or-later
 
 
+import os
 import unittest
 from pathlib import Path
 
@@ -27,6 +28,7 @@ gi.require_version("Gtk", "4.0")
 from gi.repository import Gtk
 
 from blueprintcompiler import decompiler, parser, tokenizer, utils
+from blueprintcompiler.ast_utils import AstNode
 from blueprintcompiler.completions import complete
 from blueprintcompiler.errors import (
     CompileError,
@@ -36,7 +38,6 @@ from blueprintcompiler.errors import (
 )
 from blueprintcompiler.lsp import LanguageServer
 from blueprintcompiler.outputs.xml import XmlOutput
-from blueprintcompiler.tokenizer import Token, TokenType, tokenize
 
 
 class TestSamples(unittest.TestCase):
@@ -61,11 +62,14 @@ class TestSamples(unittest.TestCase):
         except:
             pass
 
-    def assert_ast_doesnt_crash(self, text, tokens, ast):
+    def assert_ast_doesnt_crash(self, text, tokens, ast: AstNode):
+        lsp = LanguageServer()
         for i in range(len(text)):
             ast.get_docs(i)
         for i in range(len(text)):
-            list(complete(LanguageServer(), ast, tokens, i))
+            list(complete(lsp, ast, tokens, i))
+        for i in range(len(text)):
+            ast.get_reference(i)
         ast.get_document_symbols()
 
     def assert_sample(self, name, skip_run=False):
@@ -130,18 +134,20 @@ class TestSamples(unittest.TestCase):
             if len(warnings):
                 raise MultipleErrors(warnings)
         except PrintableError as e:
+            # Ignore deprecation warnings because new versions of libraries can introduce
+            # new deprecations, which would cause the tests to fail
+            errors = [
+                error
+                for error in (e.errors if isinstance(e, MultipleErrors) else [e])
+                if (name == "deprecations" or not isinstance(error, DeprecatedWarning))
+            ]
 
             def error_str(error: CompileError):
                 line, col = utils.idx_to_pos(error.range.start + 1, blueprint)
                 len = error.range.length
                 return ",".join([str(line + 1), str(col), str(len), error.message])
 
-            if isinstance(e, CompileError):
-                actual = error_str(e)
-            elif isinstance(e, MultipleErrors):
-                actual = "\n".join([error_str(error) for error in e.errors])
-            else:  # pragma: no cover
-                raise AssertionError()
+            actual = "\n".join([error_str(error) for error in errors])
 
             self.assertEqual(actual.strip(), expected.strip())
         else:  # pragma: no cover
@@ -150,17 +156,27 @@ class TestSamples(unittest.TestCase):
     def assert_decompile(self, name):
         print(f'assert_decompile("{name}")')
         try:
-            with open((Path(__file__).parent / f"samples/{name}.blp").resolve()) as f:
-                expected = f.read()
+            if os.path.exists(
+                (Path(__file__).parent / f"samples/{name}_dec.blp").resolve()
+            ):
+                with open(
+                    (Path(__file__).parent / f"samples/{name}_dec.blp").resolve()
+                ) as f:
+                    expected = f.read().strip()
+            else:
+                with open(
+                    (Path(__file__).parent / f"samples/{name}.blp").resolve()
+                ) as f:
+                    expected = f.read().strip()
 
             name = name.removesuffix("_dec")
             ui_path = (Path(__file__).parent / f"samples/{name}.ui").resolve()
 
-            actual = decompiler.decompile(ui_path)
+            actual = decompiler.decompile(ui_path).strip()
 
-            self.assertEqual(actual.strip(), expected.strip())
+            self.assertEqual(actual, expected)
         except PrintableError as e:  # pragma: no cover
-            e.pretty_print(name + ".blp", blueprint)
+            e.pretty_print(name + ".blp", expected)
             raise AssertionError()
 
     def test_samples(self):
@@ -173,31 +189,51 @@ class TestSamples(unittest.TestCase):
         samples.sort()
         for sample in samples:
             REQUIRE_ADW_1_4 = ["adw_breakpoint"]
-            REQUIRE_ADW_1_5 = ["adw_alertdialog_responses"]
+            REQUIRE_ADW_1_5 = [
+                "adw_alertdialog_responses",
+                "adw_alert_dialog_duplicate_flags",
+            ]
 
             SKIP_RUN = [
+                "adw_breakpoint_template",
                 "expr_closure",
                 "expr_closure_args",
+                "expr_value_closure",
                 "parseable",
                 "signal",
+                "signal_not_swapped",
                 "template",
                 "template_binding",
                 "template_binding_extern",
                 "template_bind_property",
                 "template_id",
                 "template_no_parent",
+                "template_orphan",
                 "template_simple_binding",
                 "typeof",
                 "unchecked_class",
             ]
 
+            # Decompiler-only tests
+            SKIP_COMPILE = ["issue_177", "translator_comments"]
+
+            SKIP_DECOMPILE = [
+                # Comments are not preserved in either direction
+                "comments",
+            ]
+
             if sample in REQUIRE_ADW_1_4 and not self.have_adw_1_4:
                 continue
             if sample in REQUIRE_ADW_1_5 and not self.have_adw_1_5:
                 continue
 
             with self.subTest(sample):
-                self.assert_sample(sample, skip_run=sample in SKIP_RUN)
+                if sample not in SKIP_COMPILE:
+                    self.assert_sample(sample, skip_run=sample in SKIP_RUN)
+
+            with self.subTest("decompile/" + sample):
+                if sample not in SKIP_DECOMPILE:
+                    self.assert_decompile(sample)
 
         # list the sample_errors directory
         sample_errors = [
@@ -205,9 +241,6 @@ class TestSamples(unittest.TestCase):
         ]
         sample_errors.sort()
         for sample_error in sample_errors:
-            REQUIRE_ADW_1_4 = ["adw_breakpoint"]
-            REQUIRE_ADW_1_5 = ["adw_alert_dialog_duplicate_flags"]
-
             if sample_error in REQUIRE_ADW_1_4 and not self.have_adw_1_4:
                 continue
             if sample_error in REQUIRE_ADW_1_5 and not self.have_adw_1_5:
@@ -215,26 +248,3 @@ class TestSamples(unittest.TestCase):
 
             with self.subTest(sample_error):
                 self.assert_sample_error(sample_error)
-
-    def test_decompiler(self):
-        self.assert_decompile("accessibility_dec")
-        if self.have_adw_1_5:
-            self.assert_decompile("adw_alertdialog_responses")
-        self.assert_decompile("adw_messagedialog_responses")
-        self.assert_decompile("child_type")
-        self.assert_decompile("file_filter")
-        self.assert_decompile("flags")
-        self.assert_decompile("id_prop")
-        self.assert_decompile("layout_dec")
-        self.assert_decompile("menu_dec")
-        self.assert_decompile("property")
-        self.assert_decompile("property_binding_dec")
-        self.assert_decompile("placeholder_dec")
-        self.assert_decompile("scale_marks")
-        self.assert_decompile("signal")
-        self.assert_decompile("strings_dec")
-        self.assert_decompile("style_dec")
-        self.assert_decompile("template")
-        self.assert_decompile("translated")
-        self.assert_decompile("using")
-        self.assert_decompile("unchecked_class_dec")
